//' Compute only single kernel evaluation.
//' 120906

#include <iostream>
#include <fstream>
#include <sys/time.h>
#include <cmath>
#include "DataType.h"
#include "ErrorCode.h"
#include "ESA.h"
#include "StringKernel.h"

using namespace std;



void help()
{
  printf("==========================================\n"
         "=    Compute single kernel evaluation    =\n"
         "==========================================\n"
         "Usage: ./file_singlek [swf] [param] file1 file2\n"
         "Note: '\\n' in file1 and file 2 will be replaced by a whitespace\n"
         "Options:\n"
         "swf (substring weighting functions)\n"
         "  0 - Constant Weight (default)\n"
         "  1 - Exponential Decay (lambda=0.5)\n"
         "  2 - K-Spectrum (k=5)\n"
         "  3 - Bounded Range (n=1)\n"
         "param (swf parameter)\n"
         "(Note that a SENTINEL character is appended to each input string.)\n\n"
         );
}


int main(int argc, char *argv[])
{
  int    swf = 0;
  Real   param = 0.0;
	Real   k1=0.0, k2=0.0, k3=0.0, k4=0.0;
  string fn1 = "";
  string fn2 = "";
  string s1  = "";
  string s2  = "";

  if(argc < 3) { help(); exit(0); }

  if(argc < 4 ) {
    //' No swf given, use default (= CONSTANT)
    swf = 0;  
    fn1 = string(argv[1]);
    fn2 = string(argv[2]);
  }
  else {
    swf = atoi(argv[1]);
    param = atof(argv[2]);
    if(swf < 0 || swf > 3) swf = 0;
    fn1 = string(argv[3]);
    fn2 = string(argv[4]);
  }
    
  
  ifstream fp1(fn1.c_str());
  ifstream fp2(fn2.c_str());

  string tmp = "";
  while(!fp1.eof()) {
    getline(fp1, tmp);
    s1 += tmp+" ";
  }
  cout << "last s1 : '" <<  s1[s1.size()-1] << "'" << endl;
  s1[s1.size()-1] = SENTINEL;

  tmp = "";
  while(!fp2.eof()) {
    getline(fp2, tmp);
    s2 += tmp+" ";
  }
  cout << "last s2 : '" <<  s2[s2.size()-1] << "'" << endl;
  s2[s2.size()-1] = SENTINEL;

  //' setup string kernels
  StringKernel sk1(s1.size(), (SYMBOL*)s1.c_str(), swf, param);
  sk1.Set_Lvs();
  sk1.PrecomputeVal();

  StringKernel sk2(s2.size(), (SYMBOL*)s2.c_str(), swf, param);
  sk2.Set_Lvs();
  sk2.PrecomputeVal();

  //' kernel evaluations
  sk1.Compute_K((SYMBOL*)s1.c_str(), s1.size()-1, k1);  // ignore SENTINEL
  sk2.Compute_K((SYMBOL*)s2.c_str(), s2.size()-1, k2);  // ignore SENTINEL
  sk1.Compute_K((SYMBOL*)s2.c_str(), s2.size()-1, k3);  // ignore SENTINEL
  sk2.Compute_K((SYMBOL*)s1.c_str(), s1.size()-1, k4);  // ignore SENTINEL


  //' dump output
  cout << "file1     : " << fn1 <<  "  (size:" << s1.size() <<")"<<endl;
  cout << "file2     : " << fn2 <<  "  (size:" << s2.size() <<")"<<endl;
  cout << "swf       : " << swf << endl;
  cout << "swf param : " << param << endl;
  cout.setf(ios::fixed);
  cout << "k(s1,s1)  : " << k1 << endl;
  cout << "k(s2,s2)  : " << k2 << endl;
  cout << "k(s1,s2)  : " << k3 << endl;
  cout << "k(s2,s1)  : " << k4 << endl;
  cout << "normalised k(s1,s2) : " << k3/sqrt(k1*k2) << endl;

	return 0;
}
